@@ -2,7 +2,7 @@
 #
 # Version
 #      $Source: D:/src/perl/File/Repl/RCS/Repl.pm $
-#      $Revision: 1.14 $
+#      $Revision: 1.17 $
 #      $State: Exp $
 #
 # Start comments/code here - will not be processed into manual pages
@@ -11,6 +11,16 @@
 #
 # Revision history:
 #      $Log: Repl.pm $
+#      Revision 1.17  2001/08/03 09:38:29  Dave.Roberts
+#      corrected code error (lines 572/3) where $$ was incorrectly used
+#      corrected code error (lines 572/3) where $$ was incorrectly used in truncation code
+#
+#      Revision 1.16  2001/08/02 22:09:02  Dave.Roberts
+#      corrected code for the Rename routine
+#
+#      Revision 1.15  2001/07/17 21:05:43  Dave.Roberts
+#      small changes to _arraysort - simplifying code
+#
 #      Revision 1.14  2001/07/12 21:51:50  jj768
 #      additional documentation - and minor code changes
 #
@@ -114,7 +124,7 @@ our @EXPORT = qw(
 	
 );
 
-our $VERSION = sprintf("%d.%d", q$Revision: 1.14 $ =~ /(\d+)\.(\d+)/);
+our $VERSION = sprintf("%d.%d", q$Revision: 1.17 $ =~ /(\d+)\.(\d+)/);
 
 # Preloaded methods go here.
 #---------------------------------------------------------------------
@@ -259,9 +269,9 @@ sub _generic {
   if ($caller eq "Update") {
     if ( scalar(@_) == 4 ) {
       ($r_con,$regex,$mode,$commit) = @_;
-      }elsif ( scalar(@_) == 5 ) {
+    }elsif ( scalar(@_) == 5 ) {
       ($r_con,$regex,$negregex,$mode,$commit) = @_;
-      }else{
+    }else{
       carp ("Call the Update method with the right arguments !\n\t\$ref->Update(regex, [noregex,] action, commit)");
       print scalar(@_), " Args called ( @_ )\n";
       return;
@@ -327,9 +337,13 @@ sub _generic {
   
 # Sort files using regex and negregex
   benchmark("init") if $r_con->{bmark};
-  ($tfiles,$common,$aonly,$bonly,$amatch,$bmatch,$refa,$refb,$refatype,$refbtype) =
+  ($tfiles,$common,$aonly,$bonly,$amatch,$bmatch) =
     _arraysort($r_con, $regex, $negregex, $r_con->{nocase}) ;
   benchmark("match files") if $r_con->{bmark};
+  $refa     = $r_con->{alist};
+  $refatype = $r_con->{atype};
+  $refb     = $r_con->{blist};
+  $refbtype = $r_con->{btype};
 #****************************************************************
 # sub to copy files and build directory structure
 #****************************************************************
@@ -559,8 +573,8 @@ if ( $caller eq "Update" ) {
   foreach (@$tfiles) {
     $afile = $refa->{$_} ? $r_con->{dira} . $_ : undef;
     $bfile = $refb->{$_} ? $r_con->{dirb} . $_ : undef;
-    &$truncate(\$afile, \$$refa->{$_}) if ($afile && -s $afile);
-    &$truncate(\$bfile, \$$refb->{$_}) if ($bfile && -s $bfile);
+    &$truncate(\$afile, \$refa->{$_}) if ($afile && -s $afile);
+    &$truncate(\$bfile, \$refb->{$_}) if ($bfile && -s $bfile);
     delete $refa->{$_}, delete $refatype->{$_} if ($afile && (($refa->{$_} + $ttl) < time) && &$del($afile));
     delete $refb->{$_}, delete $refbtype->{$_} if ($bfile && (($refb->{$_} + $ttl) < time) && &$del($bfile));
   }
@@ -657,26 +671,32 @@ if ( $caller eq "Update" ) {
       }
     }
   }
+
   }elsif( $caller eq "Rename" ) {
-  foreach my $f (@$amatch) {
-    next unless exists $refa->{$f};
-    my($newname) = $f;
-    if ( $nsub =~ /^([.])(.*)($1)(.*)($1)(.*)$/ ) {
-      my($sep,$match,$replace,$arg) = ($1,$2,$4,$5);
-      $newname =~ s/$match/$match/;
-      $afile  = $r_con->{dira} . $f;
-      my ($Afile) = $r_con->{dira} . $newname;
-      if (&$mv($afile,$Afile)){
-        $refa->{$Afile}     = $refa->{$f};
-        $refatype->{$Afile} = $refatype->{$f};
-        delete $refa->{$f};
-        delete $refatype->{$f};
-      }
+    $nsub =~ m/^(.)/;
+    my($sep) = ($1);
+    if ( $nsub =~ m/^$sep(.*)$sep(.*)$sep(.*)?$/ ) {
+      my($match,$replace,$arg) = ($1,$2,$3);
+#      print "nsub $nsub\n";
+#      print "sep: $sep\nmatch : $match\n replace: $replace\narg: $arg\n";
+      foreach my $f (@$amatch) {
+        next unless exists $refa->{$f};
+        my($newname) = $f;
+        $newname =~ s/$match/$replace/;
+        next if ($newname eq $f); # next file if no change...
+        $afile  = $r_con->{dira} . $f;
+        my ($Afile) = $r_con->{dira} . $newname;
+        if (&$mv($afile,$Afile)){
+          $refa->{$Afile}     = $refa->{$f};
+          $refatype->{$Afile} = $refatype->{$f};
+          delete $refa->{$f};
+          delete $refatype->{$f};
+        }
+      }        
     }else{
-      carp "unable to understand substition argument $nsub\n";
+      carp "unable to understand substition  argument $nsub\n";
     }
   }
-}
 
 #  add references to allow @aonly, @bonly etc to be recalled from the reference
 my($retval);
@@ -696,51 +716,36 @@ return New @_;
 }
 #=====================================================================
 sub _arraysort {
-my($r_con, $regex, $negregex, $nocase) = @_;
-my(@amatch,@bmatch,@tfiles,%common,@aonly,@bonly,@temp,$name,$mtime,$type);
-my(@sorted_amatch,@sorted_bmatch,$aName,$bName,$aIndex,$bIndex);
-my %dup      = ();
-my $verbose  = $r_con->{verbose};
-my $refa     = $r_con->{alist};
-my $refatype = $r_con->{atype};
-my $refb     = $r_con->{blist};
-my $refbtype = $r_con->{btype};
-
-# If dealing with case insensitive filesystems, use regex
-# extention (?i) on pattern matching    (NH181200)
-my $regexextn = $nocase ? '(?i)' : '';
+  my($r_con, $regex, $negregex, $nocase) = @_;
+  my(@amatch,@bmatch,@tfiles,%common,@aonly,@bonly,@temp,$name,$mtime,$type);
+  my(@sorted_amatch,@sorted_bmatch,$aName,$bName,$aIndex,$bIndex);
+  my %dup      = ();
+  my $refa     = $r_con->{alist};
+  my $refatype = $r_con->{atype};
+  my $refb     = $r_con->{blist};
+  my $refbtype = $r_con->{btype};
+  my $regexextn = $nocase ? '(?i)' : '';  # use regex extention (?i) if working with case insensitive file systems
 
 # Find files matching the regex in dira
-print "Files Matching regex in $r_con->{dira}:\n" if ($verbose >= 4);
-foreach $name (keys %$refa) {
-  if ( $name && ($name =~ /$regexextn$regex/) && ($name !~ /$regexextn$negregex/) ) {
-    push (@amatch,$name);
-    if ($verbose >= 4) {
-      $mtime = %$refa->{$name};
-      $type  = %$refatype->{$name};
-      print "  $mtime  $type  $name\n";
+  print "Files Matching regex in $r_con->{dira}:\n" if ($r_con->{verbose} >= 4);
+  foreach $name (keys %$refa) {
+    if ( $name && ($name =~ /$regexextn$regex/) && ($name !~ /$regexextn$negregex/) ) {
+      push (@amatch,$name);
+      printf"  %s  %s  %s\n",%$refa->{$name},%$refatype->{$name},$name if ($r_con->{verbose} >= 4);
     }
   }
-}
 # Find files matching the regex in dirb
-print "Files Matching regex in $r_con->{dirb}:\n" if ($verbose >= 4);
-foreach $name (keys %$refb) {
-  if ( $name && $name =~ /$regexextn$regex/ && $name !~ /$regexextn$negregex/ ) {
-    push (@bmatch,$name);
-    if ($verbose >= 4) {
-      $mtime = %$refb->{$name};
-      $type  = %$refbtype->{$name};
-      print "  $mtime  $type  $name\n";
+  print "Files Matching regex in $r_con->{dirb}:\n" if ($r_con->{verbose} >= 4);
+  foreach $name (keys %$refb) {
+    if ( $name && $name =~ /$regexextn$regex/ && $name !~ /$regexextn$negregex/ ) {
+      push (@bmatch,$name);
+      printf"  %s  %s  %s\n",%$refb->{$name},%$refbtype->{$name},$name if ($r_con->{verbose} >= 4);
     }
   }
-}
-
-# Build a list of files that have an added extension ".remove"
-# These indicate tombstoned files that will be deleted from
-# all replications.     (NH261100)
-@tfiles =     grep { /.+\.remove$/i } @amatch;  # get alist files
-push @tfiles, grep { /.+\.remove$/i } @bmatch;  # get blist files
-@tfiles =     grep { ! $dup{$_} ++  } @tfiles;  # remove dups
+# Build a list of files that have an added extension ".remove" - to be deleted by tombstone routines later
+  @tfiles =     grep { /.+\.remove$/i } @amatch;  # get alist files/dirs with .remove extension
+  push @tfiles, grep { /.+\.remove$/i } @bmatch;  # get blist files/dirs with .remove extension
+  @tfiles =     grep { ! $dup{$_} ++  } @tfiles;  # remove duplicates
 
 # Find elements that are common/unique to @amatch and @bmatch
 # -put in sorted order so that we can create directories/files
@@ -750,60 +755,55 @@ push @tfiles, grep { /.+\.remove$/i } @bmatch;  # get blist files
 # On non-case sensitive filesystems (e.g Bill's) ignore the case
 # when testing for matching files.  This will still allow repl to
 # create / update files maintaining their original case. (NH311000)
-$aIndex = 0;
-$bIndex = 0;
-@sorted_amatch = $nocase ? sort {lc($a) cmp lc($b) } @amatch : sort @amatch;
-@sorted_bmatch = $nocase ? sort {lc($a) cmp lc($b) } @bmatch : sort @bmatch;
+  $aIndex = 0;
+  $bIndex = 0;
+  @sorted_amatch = $nocase ? sort {lc($a) cmp lc($b) } @amatch : sort @amatch;
+  @sorted_bmatch = $nocase ? sort {lc($a) cmp lc($b) } @bmatch : sort @bmatch;
 
-while ( $aIndex < @sorted_amatch ) {
-  last unless defined $sorted_bmatch[$bIndex];  # End of b list
-  $aName = $sorted_amatch[$aIndex];
-  $bName = $sorted_bmatch[$bIndex];
-  if ($aName eq $bName || ($nocase && lc($aName) eq lc($bName))) {
-    $common{$aName} = $bName;  # Store $aName as key and $bName as value
-    $aIndex++;
-    $bIndex++;
+  while ( $aIndex < @sorted_amatch ) {
+    last unless defined $sorted_bmatch[$bIndex];  # End of b list
+    $aName = $sorted_amatch[$aIndex];
+    $bName = $sorted_bmatch[$bIndex];
+    if ($aName eq $bName || ($nocase && lc($aName) eq lc($bName))) {
+      $common{$aName} = $bName;  # Store $aName as key and $bName as value
+      $aIndex++;
+      $bIndex++;
+    }elsif (($nocase && lc($aName) lt lc($bName)) || (!$nocase && $aName lt $bName)) {
+      push(@aonly,$aName);
+      $aIndex++;
+    }else{
+      push(@bonly,$bName);
+      $bIndex++;
+    }
   }
-  elsif (($nocase && lc($aName) lt lc($bName)) || (!$nocase && $aName lt $bName)) {
-    push(@aonly,$aName);
-    $aIndex++;
-  }
-  else {
-    push(@bonly,$bName);
-    $bIndex++;
-  }
-}
 # Get any remainder of 'a' list
-while ( $aIndex < @sorted_amatch ) {
-  push(@aonly,$sorted_amatch[$aIndex++]);
-}
+  while ( $aIndex < @sorted_amatch ) {
+    push(@aonly,$sorted_amatch[$aIndex++]);
+  }
 # Get any remainder of 'b' list
-while ($bIndex < @sorted_bmatch) {
-  push(@bonly,$sorted_bmatch[$bIndex++]);
-}
-
-# Sort @aonly and @bonly - so that directories are allways processed after files they contain
-# This way we delete the directory only after its empty
-
-@aonly = reverse sort @aonly;
-@bonly = reverse sort @bonly;
-
-if ( $verbose >= 3 ) {
-  print "Common Files :\n";
-  foreach (keys %common) {
-    print "  $_\n  $common{$_}\n";
+  while ($bIndex < @sorted_bmatch) {
+    push(@bonly,$sorted_bmatch[$bIndex++]);
   }
-  print "A dir only Files :\n";
-  foreach (@aonly) {
-    print "  $_\n";
+
+  @aonly = reverse sort @aonly; # Sort so that file preceed directories - allows deltree to delete
+  @bonly = reverse sort @bonly; # files before directories
+
+  if ( $r_con->{verbose} >= 3 ) {
+    print "Common Files :\n";
+    foreach (keys %common) {
+      print "  $_\n  $common{$_}\n";
+    }
+    print "A dir only Files :\n";
+    foreach (@aonly) {
+      print "  $_\n";
+    }
+    print "B dir only Files :\n";
+    foreach (@bonly) {
+      print "  $_\n";
+    }
+    print "Replicating ...\n";
   }
-  print "B dir only Files :\n";
-  foreach (@bonly) {
-    print "  $_\n";
-  }
-  print "Replicating ...\n";
-}
-return (\@tfiles, \%common, \@aonly, \@bonly, \@amatch, \@bmatch, $refa,$refb,$refatype,$refbtype);
+  return (\@tfiles, \%common, \@aonly, \@bonly, \@amatch, \@bmatch);
 }
 #=====================================================================
 #  If this is called with with "init" argument this initialises global variable @times
@@ -1274,6 +1274,16 @@ out of the use of the script.
 =head1 CHANGE HISTORY
 
 $Log: Repl.pm $
+Revision 1.17  2001/08/03 09:38:29  Dave.Roberts
+corrected code error (lines 572/3) where $$ was incorrectly used
+corrected code error (lines 572/3) where $$ was incorrectly used in truncation code
+
+Revision 1.16  2001/08/02 22:09:02  Dave.Roberts
+corrected code for the Rename routine
+
+Revision 1.15  2001/07/17 21:05:43  Dave.Roberts
+small changes to _arraysort - simplifying code
+
 Revision 1.14  2001/07/12 21:51:50  jj768
 additional documentation - and minor code changes
 
